@@ -77,7 +77,7 @@ class EQU:
             ## Time grid of equilibrium shotfile
             self.time = self.sf.getobject('time')
             nt = len(self.time)
-            parmv = self.sf.sfh['PARMV'].pars
+            parmv = self.sf.sf['PARMV'].data
             nR = parmv['M'] + 1
             nZ = parmv['N'] + 1
             ## R of PFM cartesian grid [m]
