@@ -1,5 +1,5 @@
-import os, logging, traceback, datetime
-from struct import unpack, pack
+import os, logging, traceback
+from struct import unpack
 import numpy as np
 from aug_sfutils import sfmap, sfobj, str_byt
 from aug_sfutils.sfmap import ObjectID as oid
@@ -17,9 +17,8 @@ logger.setLevel(logging.INFO)
 #logger.setLevel(logging.DEBUG)
 
 n_rel = 8
-SFHbuflen = 128000 # 128 kB
-PPGCLOCK  = [1e-6, 1e-5, 1e-4, 1e-3]
-LOGICAL   = sfmap.typeMap('descr', 'SFfmt', 'LOGICAL')
+PPGCLOCK = [1e-6, 1e-5, 1e-4, 1e-3]
+LOGICAL  = sfmap.typeMap('descr', 'SFfmt', 'LOGICAL')
 
 
 def getChunk(fname, start, length):
@@ -34,7 +33,7 @@ def getChunk(fname, start, length):
 class SF_READ(dict):
 
 
-    def __init__(self, sfpath, shot):
+    def __init__(self, sfpath):
 
         self.__dict__ = self
 
@@ -44,15 +43,13 @@ class SF_READ(dict):
 
         self.properties = type('', (), {})()
         self.properties.path = sfpath
-        self.properties.shot = shot
 
         self.read_sfh()
         self.set_attributes()
 
 
     def read_sfh(self, psets=True):
-
-        """Reads a full shotfile header, including the content of ParamSets and Devices"""
+        """Reads a full shotfile header, including the data of ParamSets, Devices, Lists"""
 
         sfile = self.properties.path
 
@@ -62,14 +59,6 @@ class SF_READ(dict):
         self.properties.SFobjects = []
         self.properties.addrlen = 1
 
-# Read first 128kB of binary shotfile
-        byt_str = getChunk(self.properties.path, 0, SFHbuflen)
-        if len(byt_str) < 128:
-            logger.error('Error: shotfile %s has < 128 bytes, ignored', self.properties.file)
-            self.properties.status = None
-            return
-        self.properties.byt_str = byt_str
-
         n_max = 1000
         n_obj = n_max
         self.properties.related = []
@@ -79,6 +68,7 @@ class SF_READ(dict):
                 sfo.objid = j
                 onam = str_byt.to_str(sfo.objectName.strip())
                 if sfo.objectType == oid.Diagnostic:
+                    self.properties.shot = sfo.shot_nr
                     if n_obj == n_max: # There might be several diags objects in a SFH
                         n_obj = sfo.num_objs
                 self.properties.related.append(onam)
@@ -97,18 +87,19 @@ class SF_READ(dict):
 
 
     def set_attributes(self):
+        """Sets useful context info for the entire shotfile"""
 
         for sfo in self.properties.SFobjects:
             sfo.address *= self.properties.addrlen
             sfo.relations  = [self.properties.related[jid]   for jid in sfo.rel if jid != 65535]
             sfo.relobjects = [self.properties.SFobjects[jid] for jid in sfo.rel if jid != 65535]
+            if hasattr(sfo, 'dataFormat'):
+                if sfo.dataFormat in sfmap.dataTypes['SFfmt'].values:
+                    sfo.dataType = sfmap.typeMap('SFfmt', 'descr', sfo.dataFormat)
             if sfo.objectType == oid.List:
                 sfo.getData()
-                try:
-                    sfo.data = [self.properties.related[jid] for jid in sfo.data]
-                except:
-                    traceback.print_exc()
-                    sfo.data = []
+                sfo.data = [self.properties.related[jid] for jid in sfo.data]
+
             elif sfo.objectType in (oid.Device, oid.ParamSet):
                 sfo.getData()
                 if sfo.objectType in (oid.Device, ):
@@ -116,6 +107,7 @@ class SF_READ(dict):
                         ts06 = sfo.data['TS06']
                         if ts06 > 1e15:
                             sfo.ts06 = ts06
+
             elif sfo.objectType in (oid.Signal, oid.SignalGroup):
                 for jrel, robj in enumerate(sfo.relobjects):
 # check where the related timebase is
@@ -136,19 +128,16 @@ class SF_READ(dict):
 
 
 class SF_OBJECT:
-    """Reads a generic object's metadata from the SFH's 128byte string"""
+    """Reads the metadata of a generic SF object (sfo) from the SFH's 128byte string.
+    For data, call getData()"""
 
 
     def __init__(self, jobj, properties):
 
         self.properties = properties
         self.sfname = self.properties.path
-        sfh_byte_str = getChunk(self.sfname, 0, SFHbuflen)
-        if len(sfh_byte_str) < 128:
-            logger.error('Error: shotfile %s has < 128 bytes, ignored', self.sfname)
-            return
+        byte_str = getChunk(self.sfname, jobj*128, 128)
 
-        byte_str = sfh_byte_str[jobj*128: (jobj+1)*128]
         self.objectName = str_byt.to_str(byte_str[0:8])
 
         if not self.objectName:
@@ -165,7 +154,7 @@ class SF_OBJECT:
                 break
         self.errcode  = unpack('>h', byte_str[14: 16])[0]
         fmt = '>%dH' %n_rel
-        self.rel      = list(unpack(fmt,  byte_str[16        : 16+2*n_rel]))
+        self.rel = list(unpack(fmt,  byte_str[16 : 16+2*n_rel]))
         self.address, self.length = unpack('>2I', byte_str[16+2*n_rel: 24+2*n_rel])
         val = byte_str[40:  64]
         logger.debug('%s %d %d', self.objectName, self.address, self.length)
@@ -225,9 +214,9 @@ class SF_OBJECT:
             self.date     = val[16: 24]
 
         elif typ == oid.UpdateSet:
-            self.nitems =  unpack('>H' , val[ 2: 4])[0]
-            self.input_vals = unpack('>i' , val[ 4: 8])[0]
-            self.next_index, self.size = unpack('>I' , val[ 16: 24])
+            self.nitems     = unpack('>H', val[ 2: 4])[0]
+            self.input_vals = unpack('>i', val[ 4: 8])[0]
+            self.next_index, self.size = unpack('>I', val[ 16: 24])
 
         elif typ == oid.LocTimer:
             self.dataFormat, self.resolution = unpack('>2H', val[: 4])
@@ -235,18 +224,18 @@ class SF_OBJECT:
 
         elif typ == oid.AreaBase:
             self.dataFormat = unpack('>H' , val[ : 2])[0]
-            self.physunit    = unpack('>3H', val[2: 8])
+            self.physunit   = unpack('>3H', val[2: 8])
             self.phys_unit = [sfmap.unit_d[x] for x in self.physunit]
-            self.size_x, self.size_y, self.size_z, self.n_steps = unpack('>4I' , val[8: 24])
+            self.size_x, self.size_y, self.size_z, self.n_steps = unpack('>4I', val[8: 24])
             self.sizes = [self.size_x, self.size_y, self.size_z]
 
         elif typ == oid.Qualifier:
-            self.dataFormat = unpack('>H' , val[ : 2])[0]
-            self.num_dims, self.qsub_typ = unpack('>2H' , val[4: 8])
-            self.index_4, self.index_3, self.index_2, self.max_sections = unpack('>4I' , val[8: 24])
+            self.dataFormat = unpack('>H', val[ : 2])[0]
+            self.num_dims, self.qsub_typ = unpack('>2H', val[4: 8])
+            self.index_4, self.index_3, self.index_2, self.max_sections = unpack('>4I', val[8: 24])
 
         elif typ == oid.ModObj:
-            self.nitems = unpack('>H' , val[ : 2])[0]
+            self.nitems = unpack('>H', val[ : 2])[0]
 
         elif typ == oid.MapExtd:
             self.nitems, self.mapalg = unpack('>2H', val[ :  4])
@@ -255,23 +244,25 @@ class SF_OBJECT:
             val_2, val_3  = unpack('>2I', val[16: 24])
 
         elif typ == oid.Resource:
-            self.num_cpus, self.first_cpu = unpack('>2H' , val[ : 4])
+            self.num_cpus, self.first_cpu = unpack('>2H', val[ : 4])
 
         elif typ == oid.ADDRLEN:
-            self.addrlen = unpack('>H' , val[ : 2])[0]
+            self.addrlen = unpack('>H', val[ : 2])[0]
 
 
-    def getData(self, nbeg=None, nend=None):
+    def getData(self, nbeg=0, nend=None):
+        """Stores the data part of a SF object into sfo.data"""
 
         if self.objectType in (oid.ParamSet, oid.Device):
             self.getParamSet()
         elif self.objectType == oid.List:
             self.getList()
         elif self.objectType in (oid.TimeBase, oid.AreaBase, oid.Signal, oid.SignalGroup):
-            self.getObject(nbeg=None, nend=None)
+            self.getObject(nbeg=nbeg, nend=nend)
 
 
     def getList(self):
+        """Stores the object IDs contained in a SF list (such as SIGNALS)"""
 
         buf = getChunk(self.sfname, self.address, self.length)
         sfmt = sfmap.typeMap('SFfmt', 'struc', self.dataFormat)
@@ -302,36 +293,36 @@ class SF_OBJECT:
             j0 += 16
 
             if dfmt in sfmap.fmt2len.keys(): # char variable
-                    dlen = sfmap.fmt2len[dfmt]
-                    bytlen = n_items * dlen
-                    meta.dmin = buf[j0  : j0+1]
-                    meta.dmax = buf[j0+1: j0+2]
-                    if len(meta.dmin) == 0:
-                        meta.dmin = b' '
-                    if len(meta.dmax) == 0:
-                        meta.dmax = b' '
-                    data = np.chararray((n_items,), itemsize=dlen, buffer=buf[j0+2: j0+2+bytlen])
-                    dj0 = 8 * ( (bytlen + 9)//8 )
-                    j0 += dj0
+                dlen = sfmap.fmt2len[dfmt]
+                bytlen = n_items * dlen
+                meta.dmin = buf[j0  : j0+1]
+                meta.dmax = buf[j0+1: j0+2]
+                if len(meta.dmin) == 0:
+                    meta.dmin = b' '
+                if len(meta.dmax) == 0:
+                    meta.dmax = b' '
+                data = np.chararray((n_items,), itemsize=dlen, buffer=buf[j0+2: j0+2+bytlen])
+                dj0 = 8 * ( (bytlen + 9)//8 )
+                j0 += dj0
             elif dfmt in sfmap.dataTypes['SFfmt'].values:
-                    sfmt = sfmap.typeMap('SFfmt', 'struc', dfmt)
-                    logger.debug('Numerical par %d', dfmt)
-                    val_len = n_items + 2
-                    bytlen = val_len * np.dtype(sfmt).itemsize
-                    if n_items >= 0:
-                        if dfmt == LOGICAL: # Logical, bug if n_items > 1?
-                            meta.dmin = unpack(sfmt, buf[j0  : j0+1])[0]
-                            meta.dmax = unpack(sfmt, buf[j0+3: j0+4])[0]
-                            data = unpack(sfmt, buf[j0+5: j0+6])[0]
-                        else:
-                            data = np.ndarray((val_len, ), '>%s' %sfmt, buf[j0: j0+bytlen], order='F').copy()
-                            meta.dmin = data[0]
-                            meta.dmax = data[1]
-                            data = np.squeeze(data[2:]) # no array if n_items=1
-                    dj0 = str_byt.next8(bytlen)
-                    j0 += dj0
+                sfmt = sfmap.typeMap('SFfmt', 'struc', dfmt)
+                logger.debug('Numerical par %d', dfmt)
+                val_len = n_items + 2
+                bytlen = val_len * np.dtype(sfmt).itemsize
+                if n_items >= 0:
+                    if dfmt == LOGICAL: # Logical, bug if n_items > 1?
+                        meta.dmin = unpack(sfmt, buf[j0  : j0+1])[0]
+                        meta.dmax = unpack(sfmt, buf[j0+3: j0+4])[0]
+                        data = unpack(sfmt, buf[j0+5: j0+6])[0]
+                    else:
+                        data = np.ndarray((val_len, ), '>%s' %sfmt, buf[j0: j0+bytlen], order='F').copy()
+                        meta.dmin = data[0]
+                        meta.dmax = data[1]
+                        data = np.squeeze(data[2:]) # no array if n_items=1
+                dj0 = str_byt.next8(bytlen)
+                j0 += dj0
             else: # faulty dfmt
-                    break
+                break
 
             self.data[pname] = sfobj.SFOBJ(data, sfho=meta)
 
@@ -339,7 +330,8 @@ class SF_OBJECT:
                 break
 
 
-    def getObject(self, nbeg=None, nend=None):
+    def getObject(self, nbeg=0, nend=None):
+        """Stores data part of Sig, SigGrou, TimeBase, AreaBase"""
 
         otyp = self.objectType
         if hasattr(self, 'nbeg'):
@@ -359,7 +351,6 @@ class SF_OBJECT:
             return None
 
         dfmt = self.dataFormat
-        logger.debug('TB %s %d %d %d', self.objectName, otyp, self.length, dfmt)
         if otyp == oid.TimeBase and self.length == 0:
             if self.tbase_type == sfmap.TimebaseType.PPG_prog: # e.g. END:T-LM_END
                 self.ppg_time()
@@ -376,19 +367,17 @@ class SF_OBJECT:
                 addr = self.address
                 bytlen = np.prod(shape_arr) * type_len
                 if otyp in (oid.Signal, oid.TimeBase, oid.AreaBase) or self.time_last():
-                    if nbeg is None:
-                        nbeg = 0
-                    else:
-                        byt_beg = nbeg*np.prod(shape_arr[:-1])
-                        addr   += byt_beg*type_len
-                        bytlen -= byt_beg*type_len
-                        shape_arr[-1] -= nbeg
+                    byt_beg = nbeg*np.prod(shape_arr[:-1])
+                    addr   += byt_beg*type_len
+                    bytlen -= byt_beg*type_len
+                    shape_arr[-1] -= nbeg
                     if nend is not None:
                         bytlen = (nend - nbeg)*np.prod(shape_arr[:-1])*type_len
                         shape_arr[-1] = nend - nbeg
 
                 self.data = np.ndarray(shape_arr, '>%s' %sfmt, getChunk(self.sfname, addr, bytlen), order='F')
 
+
     def ppg_time(self): # Bug MAG:27204
         """Returns the time-array in [s] for TB of type PPG_prog"""
 
@@ -425,43 +414,9 @@ class SF_OBJECT:
                     self.data = time_ppg[:nsteps]
 
 
-    def time_first(self):
-        """True if SigGroup has time as first coordinate"""
-
-        if self.objectType != oid.SignalGroup:
-            return False
-
-        return (self.time_dim == 0)
-
-
     def time_last(self):
         """True if SigGroup has time as last coordinate"""
 
-        if self.objectType != oid.SignalGroup:
-            return False
         if not hasattr(self, 'time_dim'):
             return False
         return (self.time_dim == self.num_dims-1)
-
-
-if __name__ == '__main__':
-
-    sfile = '/afs/ipp/home/a/augd/shots/2805/L1/CEZ/28053.3'
-    nshot = 28053
-    diag = 'CEZ'
-    cez = SF_READ(diag, nshot)
-    print(cez.properties.time)
-    print(cez.Ti.phys_unit)
-    print(cez.properties.parsets)
-    print(cez.LOSInfo.__dict__)
-    cez.Ti.getData()
-    titim = cez.Ti.timebase
-    titim.getData()
-    tiarea = cez.Ti.areabase
-    tiarea.getData()
-    print(cez.Ti.data[0, :])
-    print(titim.data[:50])
-    print(tiarea.data.shape)
-    print(cez.Ti.areabase.objectName)
-    print(cez.Ti.timebase.objectName)
-    print(cez.SIGNALS.data)
