@@ -1,13 +1,10 @@
 import os, datetime, logging
-from struct import unpack
 import numpy as np
 
 from aug_sfutils import sfmap, sfobj, manage_ed, parse_kwargs, str_byt, libddc, getlastshot
 from aug_sfutils.sf_read import SF_READ
 from aug_sfutils.sfmap import ObjectID as oid
 
-PPGCLOCK = [1e-6, 1e-5, 1e-4, 1e-3]
-
 logger = logging.getLogger('aug_sfutils.sfread')
 date_fmt = '%Y-%m-%d'
 logger.setLevel(logging.INFO)
@@ -126,7 +123,7 @@ class SFREAD:
             return
 
         logger.info('Fetching SF %s', self.sfpath)
-        self.sf = SF_READ(self.sfpath, self.shot)
+        self.sf = SF_READ(self.sfpath)
         self.status = (self.sf is not None)
 
         self.cache = {}
@@ -189,7 +186,7 @@ class SFREAD:
         return None
 
 
-    def getobject(self, obj, cal=True, nbeg=None, nend=None, tbeg=None, tend=None):
+    def getobject(self, obj, cal=True, nbeg=0, nend=None, tbeg=None, tend=None):
         """
         Reads the data of a given TB, AB, SIG or SGR
         """
